@@ -34,71 +34,6 @@
 #include "gdkinternals.h"	/* For scratch_image code */
 #include "gdkalias.h"
 
-/**
- * gdk_image_ref:
- * @image: a #GdkImage
- *
- * Deprecated function; use g_object_ref() instead.
- * 
- * Return value: the image
- *
- * Deprecated: 2.0: Use g_object_ref() instead.
- **/
-GdkImage *
-gdk_image_ref (GdkImage *image)
-{
-  g_return_val_if_fail (GDK_IS_IMAGE (image), NULL);
-
-  return g_object_ref (image);
-}
-
-/**
- * gdk_image_unref:
- * @image: a #GdkImage
- *
- * Deprecated function; use g_object_unref() instead.
- *
- * Deprecated: 2.0: Use g_object_unref() instead.
- **/
-void
-gdk_image_unref (GdkImage *image)
-{
-  g_return_if_fail (GDK_IS_IMAGE (image));
-
-  g_object_unref (image);
-}
-
-/**
- * gdk_image_get:
- * @drawable: a #GdkDrawable
- * @x: x coordinate in @window
- * @y: y coordinate in @window
- * @width: width of area in @window
- * @height: height of area in @window
- * 
- * This is a deprecated wrapper for gdk_drawable_get_image();
- * gdk_drawable_get_image() should be used instead. Or even better: in
- * most cases gdk_pixbuf_get_from_drawable() is the most convenient
- * choice.
- * 
- * Return value: a new #GdkImage or %NULL
- **/
-GdkImage*
-gdk_image_get (GdkWindow *drawable,
-	       gint       x,
-	       gint       y,
-	       gint       width,
-	       gint       height)
-{
-  g_return_val_if_fail (GDK_IS_DRAWABLE (drawable), NULL);
-  g_return_val_if_fail (x >= 0, NULL);
-  g_return_val_if_fail (y >= 0, NULL);
-  g_return_val_if_fail (width >= 0, NULL);
-  g_return_val_if_fail (height >= 0, NULL);
-  
-  return gdk_drawable_get_image (drawable, x, y, width, height);
-}
-
 /**
  * gdk_image_set_colormap:
  * @image: a #GdkImage
@@ -308,24 +243,6 @@ gdk_image_get_bits_per_pixel (GdkImage *image)
   return image->bits_per_pixel;
 }
 
-/**
- * gdk_image_get_pixels:
- * @image: a #GdkImage
- *
- * Returns a pointer to the pixel data of the image.
- *
- * Returns: the pixel data of the image
- *
- * Since: 2.22
- */
-gpointer
-gdk_image_get_pixels (GdkImage *image)
-{
-  g_return_val_if_fail (GDK_IS_IMAGE (image), NULL);
-
-  return image->mem;
-}
-
 /* We have N_REGION GDK_SCRATCH_IMAGE_WIDTH x GDK_SCRATCH_IMAGE_HEIGHT regions divided
  * up between n_images different images. possible_n_images gives
  * various divisors of N_REGIONS. The reason for allowing this
