@@ -21,21 +21,21 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_ALIGNMENT_H__
 #define __GTK_ALIGNMENT_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkbin.h>
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_ALIGNMENT                  (gtk_alignment_get_type ())
 #define GTK_ALIGNMENT(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_ALIGNMENT, GtkAlignment))
@@ -53,10 +53,10 @@ struct _GtkAlignment
 {
   GtkBin bin;
 
-  gfloat xalign;
-  gfloat yalign;
-  gfloat xscale;
-  gfloat yscale;
+  gfloat GSEAL (xalign);
+  gfloat GSEAL (yalign);
+  gfloat GSEAL (xscale);
+  gfloat GSEAL (yscale);
 };
 
 struct _GtkAlignmentClass
@@ -88,9 +88,7 @@ void       gtk_alignment_get_padding (GtkAlignment      *alignment,
 				      guint             *padding_left,
 				      guint             *padding_right);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 
 #endif /* __GTK_ALIGNMENT_H__ */
