@@ -42,6 +42,7 @@ struct _GtkHandleBoxPrivate
 {
   gint orig_x;
   gint orig_y;
+  GdkDevice *grab_device;
 };
 
 enum {
@@ -259,7 +260,7 @@ gtk_handle_box_get_private (GtkHandleBox *hb)
 static void
 gtk_handle_box_init (GtkHandleBox *handle_box)
 {
-  GTK_WIDGET_UNSET_FLAGS (handle_box, GTK_NO_WINDOW);
+  gtk_widget_set_has_window (GTK_WIDGET (handle_box), TRUE);
 
   handle_box->bin_window = NULL;
   handle_box->float_window = NULL;
@@ -348,14 +349,14 @@ gtk_handle_box_map (GtkWidget *widget)
   GtkBin *bin;
   GtkHandleBox *hb;
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
+  gtk_widget_set_mapped (widget, TRUE);
 
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
 
   if (bin->child &&
-      GTK_WIDGET_VISIBLE (bin->child) &&
-      !GTK_WIDGET_MAPPED (bin->child))
+      gtk_widget_get_visible (bin->child) &&
+      !gtk_widget_get_mapped (bin->child))
     gtk_widget_map (bin->child);
 
   if (hb->child_detached && !hb->float_window_mapped)
@@ -373,7 +374,7 @@ gtk_handle_box_unmap (GtkWidget *widget)
 {
   GtkHandleBox *hb;
 
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
+  gtk_widget_set_mapped (widget, FALSE);
 
   hb = GTK_HANDLE_BOX (widget);
 
@@ -394,7 +395,7 @@ gtk_handle_box_realize (GtkWidget *widget)
 
   hb = GTK_HANDLE_BOX (widget);
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  gtk_widget_set_realized (widget, TRUE);
 
   attributes.x = widget->allocation.x;
   attributes.y = widget->allocation.y;
@@ -450,9 +451,9 @@ gtk_handle_box_realize (GtkWidget *widget)
   gdk_window_set_type_hint (hb->float_window, GDK_WINDOW_TYPE_HINT_TOOLBAR);
   
   widget->style = gtk_style_attach (widget->style, widget->window);
-  gtk_style_set_background (widget->style, widget->window, GTK_WIDGET_STATE (hb));
-  gtk_style_set_background (widget->style, hb->bin_window, GTK_WIDGET_STATE (hb));
-  gtk_style_set_background (widget->style, hb->float_window, GTK_WIDGET_STATE (hb));
+  gtk_style_set_background (widget->style, widget->window, gtk_widget_get_state (widget));
+  gtk_style_set_background (widget->style, hb->bin_window, gtk_widget_get_state (widget));
+  gtk_style_set_background (widget->style, hb->float_window, gtk_widget_get_state (widget));
   gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
 }
 
@@ -477,8 +478,8 @@ gtk_handle_box_style_set (GtkWidget *widget,
 {
   GtkHandleBox *hb = GTK_HANDLE_BOX (widget);
 
-  if (GTK_WIDGET_REALIZED (widget) &&
-      !GTK_WIDGET_NO_WINDOW (widget))
+  if (gtk_widget_get_realized (widget) &&
+      gtk_widget_get_has_window (widget))
     {
       gtk_style_set_background (widget->style, widget->window,
 				widget->state);
@@ -612,7 +613,7 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
       
   widget->allocation = *allocation;
 
-  if (GTK_WIDGET_REALIZED (hb))
+  if (gtk_widget_get_realized (widget))
     gdk_window_move_resize (widget->window,
 			    widget->allocation.x,
 			    widget->allocation.y,
@@ -620,7 +621,7 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
 			    widget->allocation.height);
 
 
-  if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+  if (bin->child && gtk_widget_get_visible (bin->child))
     {
       GtkAllocation child_allocation;
       guint border_width;
@@ -651,7 +652,7 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
 	  else
 	    float_height += DRAG_HANDLE_SIZE;
 
-	  if (GTK_WIDGET_REALIZED (hb))
+	  if (gtk_widget_get_realized (widget))
 	    {
 	      gdk_window_resize (hb->float_window,
 				 float_width,
@@ -674,7 +675,7 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
 	  else
 	    child_allocation.height -= DRAG_HANDLE_SIZE;
 	  
-	  if (GTK_WIDGET_REALIZED (hb))
+	  if (gtk_widget_get_realized (widget))
 	    gdk_window_move_resize (hb->bin_window,
 				    0,
 				    0,
@@ -716,7 +717,7 @@ gtk_handle_box_draw_ghost (GtkHandleBox *hb)
     }
   gtk_paint_shadow (widget->style,
 		    widget->window,
-		    GTK_WIDGET_STATE (widget),
+		    gtk_widget_get_state (widget),
 		    GTK_SHADOW_ETCHED_IN,
 		    NULL, widget, "handle",
 		    x,
@@ -727,7 +728,7 @@ gtk_handle_box_draw_ghost (GtkHandleBox *hb)
        handle_position == GTK_POS_RIGHT)
      gtk_paint_hline (widget->style,
 		      widget->window,
-		      GTK_WIDGET_STATE (widget),
+		      gtk_widget_get_state (widget),
 		      NULL, widget, "handlebox",
 		      handle_position == GTK_POS_LEFT ? DRAG_HANDLE_SIZE : 0,
 		      handle_position == GTK_POS_LEFT ? widget->allocation.width : widget->allocation.width - DRAG_HANDLE_SIZE,
@@ -735,7 +736,7 @@ gtk_handle_box_draw_ghost (GtkHandleBox *hb)
    else
      gtk_paint_vline (widget->style,
 		      widget->window,
-		      GTK_WIDGET_STATE (widget),
+		      gtk_widget_get_state (widget),
 		      NULL, widget, "handlebox",
 		      handle_position == GTK_POS_TOP ? DRAG_HANDLE_SIZE : 0,
 		      handle_position == GTK_POS_TOP ? widget->allocation.height : widget->allocation.height - DRAG_HANDLE_SIZE,
@@ -890,14 +891,14 @@ gtk_handle_box_paint (GtkWidget      *widget,
   if (!event)
     gtk_paint_box (widget->style,
 		   hb->bin_window,
-		   GTK_WIDGET_STATE (widget),
+		   gtk_widget_get_state (widget),
 		   hb->shadow_type,
 		   area, widget, "handlebox_bin",
 		   0, 0, -1, -1);
   else
    gtk_paint_box (widget->style,
 		  hb->bin_window,
-		  GTK_WIDGET_STATE (widget),
+		  gtk_widget_get_state (widget),
 		  hb->shadow_type,
 		  &event->area, widget, "handlebox_bin",
 		  0, 0, -1, -1);
@@ -951,7 +952,7 @@ gtk_handle_box_paint (GtkWidget      *widget,
 			 event ? &event->area : area,
 			 handle_orientation);
 
-  if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+  if (bin->child && gtk_widget_get_visible (bin->child))
     GTK_WIDGET_CLASS (gtk_handle_box_parent_class)->expose_event (widget, event);
 }
 
@@ -961,7 +962,7 @@ gtk_handle_box_expose (GtkWidget      *widget,
 {
   GtkHandleBox *hb;
 
-  if (GTK_WIDGET_DRAWABLE (widget))
+  if (gtk_widget_is_drawable (widget))
     {
       hb = GTK_HANDLE_BOX (widget);
 
@@ -1079,7 +1080,8 @@ gtk_handle_box_button_press (GtkWidget      *widget,
 	      gint root_x, root_y;
 	      gint width, height;
 
-              gtk_invisible_set_screen (GTK_INVISIBLE (invisible), gtk_widget_get_screen (hb));
+              gtk_invisible_set_screen (GTK_INVISIBLE (invisible),
+                                        gtk_widget_get_screen (GTK_WIDGET (hb)));
 	      gdk_window_get_deskrelative_origin (hb->bin_window, &desk_x, &desk_y);
 	      gdk_window_get_origin (hb->bin_window, &root_x, &root_y);
 	      gdk_drawable_get_size (hb->bin_window, &width, &height);
@@ -1113,22 +1115,25 @@ gtk_handle_box_button_press (GtkWidget      *widget,
 		  hb->attach_allocation.height = 0;
 		}
 	      hb->in_drag = TRUE;
+              private->grab_device = event->device;
 	      fleur = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
 						  GDK_FLEUR);
-	      if (gdk_pointer_grab (invisible->window,
-				    FALSE,
-				    (GDK_BUTTON1_MOTION_MASK |
-				     GDK_POINTER_MOTION_HINT_MASK |
-				     GDK_BUTTON_RELEASE_MASK),
-				    NULL,
-				    fleur,
-				    event->time) != 0)
+	      if (gdk_device_grab (event->device,
+                                   invisible->window,
+                                   GDK_OWNERSHIP_WINDOW,
+                                   FALSE,
+                                   (GDK_BUTTON1_MOTION_MASK |
+                                    GDK_POINTER_MOTION_HINT_MASK |
+                                    GDK_BUTTON_RELEASE_MASK),
+                                   fleur,
+                                   event->time) != GDK_GRAB_SUCCESS)
 		{
 		  hb->in_drag = FALSE;
+                  private->grab_device = NULL;
 		}
 	      else
 		{
-		  gtk_grab_add (invisible);
+                  gtk_device_grab_add (invisible, private->grab_device, TRUE);
 		  g_signal_connect (invisible, "event",
 				    G_CALLBACK (gtk_handle_box_grab_event), hb);
 		}
@@ -1168,9 +1173,10 @@ gtk_handle_box_motion (GtkWidget      *widget,
   new_x = 0;
   new_y = 0;
   screen = gtk_widget_get_screen (widget);
-  gdk_display_get_pointer (gdk_screen_get_display (screen),
-			   &pointer_screen, 
-			   &new_x, &new_y, NULL);
+  gdk_display_get_device_state (gdk_screen_get_display (screen),
+                                event->device,
+                                &pointer_screen,
+                                &new_x, &new_y, NULL);
   if (pointer_screen != screen)
     {
       GtkHandleBoxPrivate *private = gtk_handle_box_get_private (hb);
@@ -1393,7 +1399,7 @@ gtk_handle_box_reattach (GtkHandleBox *hb)
   if (hb->child_detached)
     {
       hb->child_detached = FALSE;
-      if (GTK_WIDGET_REALIZED (hb))
+      if (gtk_widget_get_realized (widget))
 	{
 	  gdk_window_hide (hb->float_window);
 	  gdk_window_reparent (hb->bin_window, widget->window, 0, 0);
@@ -1417,15 +1423,18 @@ static void
 gtk_handle_box_end_drag (GtkHandleBox *hb,
 			 guint32       time)
 {
+  GtkHandleBoxPrivate *private = gtk_handle_box_get_private (hb);
   GtkWidget *invisible = gtk_handle_box_get_invisible ();
-		
+
   hb->in_drag = FALSE;
 
-  gtk_grab_remove (invisible);
-  gdk_pointer_ungrab (time);
+  gtk_device_grab_remove (invisible, private->grab_device);
+  gdk_device_ungrab (private->grab_device, time);
   g_signal_handlers_disconnect_by_func (invisible,
 					G_CALLBACK (gtk_handle_box_grab_event),
 					hb);
+
+  private->grab_device = NULL;
 }
 
 #define __GTK_HANDLE_BOX_C__
