@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -32,7 +32,6 @@
 #define __GTK_PROGRESS_BAR_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkprogress.h>
 
 
@@ -67,21 +66,21 @@ struct _GtkProgressBar
 {
   GtkProgress progress;
 
-  GtkProgressBarStyle bar_style;
-  GtkProgressBarOrientation orientation;
+  GtkProgressBarStyle       GSEAL (bar_style);
+  GtkProgressBarOrientation GSEAL (orientation);
 
-  guint blocks;
-  gint  in_block;
+  guint GSEAL (blocks);
+  gint  GSEAL (in_block);
 
-  gint  activity_pos;
-  guint activity_step;
-  guint activity_blocks;
+  gint  GSEAL (activity_pos);
+  guint GSEAL (activity_step);
+  guint GSEAL (activity_blocks);
 
-  gdouble pulse_fraction;
+  gdouble GSEAL (pulse_fraction);
 
-  guint activity_dir : 1;
-  guint ellipsize : 3;
-  guint dirty : 1;
+  guint GSEAL (activity_dir) : 1;
+  guint GSEAL (ellipsize) : 3;
+  guint GSEAL (dirty) : 1;
 };
 
 struct _GtkProgressBarClass
@@ -147,27 +146,6 @@ void               gtk_progress_bar_set_ellipsize (GtkProgressBar     *pbar,
 PangoEllipsizeMode gtk_progress_bar_get_ellipsize (GtkProgressBar     *pbar);
 
 
-#ifndef GTK_DISABLE_DEPRECATED
-
-/* Everything below here is deprecated */
-GtkWidget* gtk_progress_bar_new_with_adjustment  (GtkAdjustment  *adjustment);
-void       gtk_progress_bar_set_bar_style        (GtkProgressBar *pbar,
-						  GtkProgressBarStyle style);
-void       gtk_progress_bar_set_discrete_blocks  (GtkProgressBar *pbar,
-						  guint           blocks);
-/* set_activity_step() is not only deprecated, it doesn't even work.
- * (Of course, it wasn't usable anyway, you had to set it from a size_allocate
- * handler or something)
- */
-void       gtk_progress_bar_set_activity_step    (GtkProgressBar *pbar,
-                                                  guint           step);
-void       gtk_progress_bar_set_activity_blocks  (GtkProgressBar *pbar,
-						  guint           blocks);
-void       gtk_progress_bar_update               (GtkProgressBar *pbar,
-						  gdouble         percentage);
-
-#endif /* GTK_DISABLE_DEPRECATED */
-
 G_END_DECLS
 
 #endif /* __GTK_PROGRESS_BAR_H__ */
