@@ -21,14 +21,17 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_TABLE_H__
 #define __GTK_TABLE_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkcontainer.h>
 
 
@@ -50,15 +53,15 @@ typedef struct _GtkTableRowCol	GtkTableRowCol;
 struct _GtkTable
 {
   GtkContainer container;
-  
-  GList *children;
-  GtkTableRowCol *rows;
-  GtkTableRowCol *cols;
-  guint16 nrows;
-  guint16 ncols;
-  guint16 column_spacing;
-  guint16 row_spacing;
-  guint homogeneous : 1;
+
+  GList *GSEAL (children);
+  GtkTableRowCol *GSEAL (rows);
+  GtkTableRowCol *GSEAL (cols);
+  guint16 GSEAL (nrows);
+  guint16 GSEAL (ncols);
+  guint16 GSEAL (column_spacing);
+  guint16 GSEAL (row_spacing);
+  guint GSEAL (homogeneous) : 1;
 };
 
 struct _GtkTableClass
