@@ -51,7 +51,7 @@
  */
 
 #define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
-#include <config.h>
+#include "config.h"
 #include "gtktextbtree.h"
 #include <string.h>
 #include <stdlib.h>
@@ -174,12 +174,12 @@ char_segment_self_check (GtkTextLineSegment *seg)
 
   if (seg->byte_count <= 0)
     {
-      g_error ("char_segment_check_func: segment has size <= 0");
+      g_error ("segment has size <= 0");
     }
 
   if (strlen (seg->body.chars) != seg->byte_count)
     {
-      g_error ("char_segment_check_func: segment has wrong size");
+      g_error ("segment has wrong size");
     }
 
   if (g_utf8_strlen (seg->body.chars, seg->byte_count) != seg->char_count)
@@ -196,7 +196,7 @@ _gtk_char_segment_new (const gchar *text, guint len)
   g_assert (gtk_text_byte_begins_utf8_char (text));
 
   seg = g_malloc (CSEG_SIZE (len));
-  seg->type = &gtk_text_char_type;
+  seg->type = (GtkTextLineSegmentClass *)&gtk_text_char_type;
   seg->next = NULL;
   seg->byte_count = len;
   memcpy (seg->body.chars, text, len);
@@ -211,8 +211,12 @@ _gtk_char_segment_new (const gchar *text, guint len)
 }
 
 GtkTextLineSegment*
-_gtk_char_segment_new_from_two_strings (const gchar *text1, guint len1,
-                                        const gchar *text2, guint len2)
+_gtk_char_segment_new_from_two_strings (const gchar *text1, 
+					guint        len1, 
+					guint        chars1,
+                                        const gchar *text2, 
+					guint        len2, 
+					guint        chars2)
 {
   GtkTextLineSegment *seg;
 
@@ -227,9 +231,7 @@ _gtk_char_segment_new_from_two_strings (const gchar *text1, guint len1,
   memcpy (seg->body.chars + len1, text2, len2);
   seg->body.chars[len1+len2] = '\0';
 
-  /* In principle this function could probably take chars1 and chars2
-     as args, since it's typically used to merge two char segments */
-  seg->char_count = g_utf8_strlen (seg->body.chars, seg->byte_count);
+  seg->char_count = chars1 + chars2;
 
   if (gtk_debug_flags & GTK_DEBUG_TEXT)
     char_segment_self_check (seg);
@@ -296,6 +298,11 @@ char_segment_split_func (GtkTextLineSegment *seg, int index)
  *      This procedure merges adjacent character segments into
  *      a single character segment, if possible.
  *
+ * Arguments:
+ *      segPtr: Pointer to the first of two adjacent segments to
+ *              join.
+ *      line:   Line containing segments (not used).
+ *
  * Results:
  *      The return value is a pointer to the first segment in
  *      the (new) list of segments that used to start with segPtr.
@@ -308,11 +315,7 @@ char_segment_split_func (GtkTextLineSegment *seg, int index)
 
         /* ARGSUSED */
 static GtkTextLineSegment *
-char_segment_cleanup_func (segPtr, line)
-     GtkTextLineSegment *segPtr;           /* Pointer to first of two adjacent
-                                         * segments to join. */
-     GtkTextLine *line;                /* Line containing segments (not
-                                        * used). */
+char_segment_cleanup_func (GtkTextLineSegment *segPtr, GtkTextLine *line)
 {
   GtkTextLineSegment *segPtr2, *newPtr;
 
@@ -326,8 +329,12 @@ char_segment_cleanup_func (segPtr, line)
     }
 
   newPtr =
-    _gtk_char_segment_new_from_two_strings (segPtr->body.chars, segPtr->byte_count,
-                                            segPtr2->body.chars, segPtr2->byte_count);
+    _gtk_char_segment_new_from_two_strings (segPtr->body.chars, 
+					    segPtr->byte_count,
+					    segPtr->char_count,
+                                            segPtr2->body.chars, 
+					    segPtr2->byte_count,
+					    segPtr2->char_count);
 
   newPtr->next = segPtr2->next;
 
@@ -346,6 +353,12 @@ char_segment_cleanup_func (segPtr, line)
  *
  *      This procedure is invoked to delete a character segment.
  *
+ * Arguments:
+ *      segPtr   : Segment to delete
+ *      line     : Line containing segment
+ *      treeGone : Non-zero means the entire tree is being
+ *                 deleted, so everything must get cleaned up.
+ *
  * Results:
  *      Always returns 0 to indicate that the segment was deleted.
  *
@@ -357,12 +370,7 @@ char_segment_cleanup_func (segPtr, line)
 
         /* ARGSUSED */
 static int
-char_segment_delete_func (segPtr, line, treeGone)
-     GtkTextLineSegment *segPtr;           /* Segment to delete. */
-     GtkTextLine *line;                /* Line containing segment. */
-     int treeGone;                      /* Non-zero means the entire tree is
-                                         * being deleted, so everything must
-                                         * get cleaned up. */
+char_segment_delete_func (GtkTextLineSegment *segPtr, GtkTextLine *line, int treeGone)
 {
   g_free ((char*) segPtr);
   return 0;
@@ -376,6 +384,10 @@ char_segment_delete_func (segPtr, line, treeGone)
  *      This procedure is invoked to perform consistency checks
  *      on character segments.
  *
+ * Arguments:
+ *      segPtr : Segment to check
+ *      line   : Line containing segment
+ *
  * Results:
  *      None.
  *
@@ -388,9 +400,7 @@ char_segment_delete_func (segPtr, line, treeGone)
 
         /* ARGSUSED */
 static void
-char_segment_check_func (segPtr, line)
-     GtkTextLineSegment *segPtr;           /* Segment to check. */
-     GtkTextLine *line;                /* Line containing segment. */
+char_segment_check_func (GtkTextLineSegment *segPtr, GtkTextLine *line)
 {
   char_segment_self_check (segPtr);
 
@@ -398,7 +408,7 @@ char_segment_check_func (segPtr, line)
     {
       if (segPtr->next->type == &gtk_text_char_type)
         {
-          g_error ("char_segment_check_func: adjacent character segments weren't merged");
+          g_error ("adjacent character segments weren't merged");
         }
     }
 }
@@ -430,6 +440,12 @@ _gtk_toggle_segment_new (GtkTextTagInfo *info, gboolean on)
  *
  *      This procedure is invoked to delete toggle segments.
  *
+ * Arguments:
+ *      segPtr   : Segment to check
+ *      line     : Line containing segment
+ *      treeGone : Non-zero means the entire tree is being
+ *                 deleted so everything must get cleaned up
+ *
  * Results:
  *      Returns 1 to indicate that the segment may not be deleted,
  *      unless the entire B-tree is going away.
@@ -443,12 +459,7 @@ _gtk_toggle_segment_new (GtkTextTagInfo *info, gboolean on)
  */
 
 static int
-toggle_segment_delete_func (segPtr, line, treeGone)
-     GtkTextLineSegment *segPtr;           /* Segment to check. */
-     GtkTextLine *line;                /* Line containing segment. */
-     int treeGone;                      /* Non-zero means the entire tree is
-                                         * being deleted, so everything must
-                                         * get cleaned up. */
+toggle_segment_delete_func (GtkTextLineSegment *segPtr, GtkTextLine *line, int treeGone)
 {
   if (treeGone)
     {
@@ -482,6 +493,10 @@ toggle_segment_delete_func (segPtr, line, treeGone)
  *      been modified in some way.  It's invoked after the
  *      modifications are complete.
  *
+ * Arguments:
+ *      segPtr : Segment to check
+ *      line   : Line that now contains segment
+ *
  * Results:
  *      The return value is the head segment in a new list
  *      that is to replace the tail of the line that used to
@@ -497,9 +512,7 @@ toggle_segment_delete_func (segPtr, line, treeGone)
  */
 
 static GtkTextLineSegment *
-toggle_segment_cleanup_func (segPtr, line)
-     GtkTextLineSegment *segPtr;   /* Segment to check. */
-     GtkTextLine *line;        /* Line that now contains segment. */
+toggle_segment_cleanup_func (GtkTextLineSegment *segPtr, GtkTextLine *line)
 {
   GtkTextLineSegment *segPtr2, *prevPtr;
   int counts;
@@ -557,6 +570,10 @@ toggle_segment_cleanup_func (segPtr, line)
  *      This procedure is invoked when a toggle segment is about
  *      to move from one line to another.
  *
+ * Arguments:
+ *      segPtr : Segment to check
+ *      line   : Line that used to contain segment
+ *
  * Results:
  *      None.
  *
@@ -567,9 +584,7 @@ toggle_segment_cleanup_func (segPtr, line)
  */
 
 static void
-toggle_segment_line_change_func (segPtr, line)
-     GtkTextLineSegment *segPtr;   /* Segment to check. */
-     GtkTextLine *line;        /* Line that used to contain segment. */
+toggle_segment_line_change_func (GtkTextLineSegment *segPtr, GtkTextLine *line)
 {
   if (segPtr->body.toggle.inNodeCounts)
     {
@@ -584,7 +599,7 @@ toggle_segment_line_change_func (segPtr, line)
  */
 
 
-GtkTextLineSegmentClass gtk_text_char_type = {
+const GtkTextLineSegmentClass gtk_text_char_type = {
   "character",                          /* name */
   0,                                            /* leftGravity */
   char_segment_split_func,                              /* splitFunc */
@@ -599,7 +614,7 @@ GtkTextLineSegmentClass gtk_text_char_type = {
  * range:
  */
 
-GtkTextLineSegmentClass gtk_text_toggle_on_type = {
+const GtkTextLineSegmentClass gtk_text_toggle_on_type = {
   "toggleOn",                                   /* name */
   0,                                            /* leftGravity */
   NULL,                 /* splitFunc */
@@ -614,7 +629,7 @@ GtkTextLineSegmentClass gtk_text_toggle_on_type = {
  * range:
  */
 
-GtkTextLineSegmentClass gtk_text_toggle_off_type = {
+const GtkTextLineSegmentClass gtk_text_toggle_off_type = {
   "toggleOff",                          /* name */
   1,                                            /* leftGravity */
   NULL,                 /* splitFunc */
